import { computed, onMounted, reactive, ref, unref } from 'vue'
import type { VxeTableEvents, VxeTableInstance } from 'vxe-table'
import {
  apiQueryUserList,
  apiInsertUser,
  apiDeleteUser,
  apiFindByIdUser,
  apiUserRoleAuth
} from '@/api/sys/user'
import type { UserInfo } from '@/api/sys/model/userModel'
import type { Pager } from '#/global'
import type { Rule } from 'ant-design-vue/es/form'
import { message } from 'ant-design-vue'
import dayjs, { Dayjs } from 'dayjs'
import { apiFindUserByRole, apiGetRole, apiGetRoleList } from '@/api/sys/role'
import type { RoleInfo } from '@/api/sys/model/roleModel'

const rules: Record<string, Rule[]> = {
  userName: [{ required: true, message: 'Please input Activity name', trigger: 'change' }],
  userPassword: [{ required: true, message: 'Please input Activity password', trigger: 'change' }]
}
const initFormState = (): UserInfo => {
  return {
    userId: '',
    userName: '',
    userRealName: '',
    userPassword: '',
    userAvatar: '',
    userBirthday: '',
    userSex: 0,
    userEmail: '',
    userPhone: '',
    userWorkNo: '',
    userIdentity: 1,
    createTime: '',
    updateTime: '',
    createBy: '',
    updateBy: '',
    orgCode: '',
    deleted: 0
  }
}

const initPagination: Pager = {
  currentPage: 1,
  total: 10,
  pageSize: 10
}

export function useUserPage() {
  const tableRef = ref<VxeTableInstance<UserInfo>>()
  const tableData = ref<UserInfo[]>([])
  const pagination = reactive<Pager>(initPagination)
  const loading = ref(false)

  /**
   * TODO:查询未添加查询条件
   */
  const queryData = async (page?: number) => {
    loading.value = true
    try {
      page && (pagination.currentPage = page)
      const { result } = await apiQueryUserList(pagination)
      tableData.value = result.records
      pagination.total = result.total
    } catch (e) {}
    loading.value = false
  }

  return {
    tableRef,
    tableData,
    pagination,
    loading,
    queryData
  }
}

export function useUserFrom() {
  const userFormRef = ref()
  const userFormRules = reactive<Record<string, Rule[]>>(rules)
  const userFormState = reactive<UserInfo>(initFormState())
  const addLoading = ref(false)

  const validate = async () => await userFormRef.value.validate()
  const resetFrom = () => Object.assign(unref(userFormState), initFormState())

  const getUserInfo = async (id: string) => {
    const { result } = await apiFindByIdUser({ id })
    Object.assign(unref(userFormState), result)
    if (userFormState.userBirthday) {
      userFormState.userBirthday = dayjs(userFormState.userBirthday as string, 'YYYY-MM-DD')
    }
  }

  const userFromSubmit = async () => {
    addLoading.value = true
    try {
      await validate()
      await apiInsertUser(userFormState)
      message.success('新增成功！')
    } catch (e) {}
    addLoading.value = false
  }

  const deleteUser = async (userIds: string) => {
    try {
      await apiDeleteUser({ userIds })
      message.success('删除成功！')
    } catch (e) {}
  }

  return {
    userFormRef,
    userFormState,
    userFormRules,
    addLoading,
    getUserInfo,
    resetFrom,
    deleteUser,
    userFromSubmit
  }
}

export function useAssignRole() {
  const selectOption = ref<(RoleInfo & { key?: string })[]>([])
  const isModal = ref(false)
  const selectValue = ref<string[]>([])
  let currentRow = reactive<UserInfo | {}>({})
  const roleColumns = ref([
    { title: '角色名称', dataIndex: 'roleName' },
    { title: '角色编码', dataIndex: 'roleCode' },
    { title: '角色描述', dataIndex: 'roleDes' }
  ])

  const roleHandle = computed(() =>
    selectValue.value.map((item) => selectOption.value.find((i) => i.roleId === item)?.roleName)
  )

  onMounted(async () => {
    const { result } = await apiGetRoleList()
    selectOption.value = result.records
    selectOption.value.forEach((item) => {
      item.key = item.roleId
    })
  })

  const onSelectChange = (keys: any[]) => {
    selectValue.value = keys
  }

  const authRole = async(row: UserInfo) => {
    isModal.value = true
    const {result } = await apiFindUserByRole({userId: row.userId as string})
    selectValue.value = result.map(item => item.roleId)
    currentRow = unref(row)
  }

  const resetState = () => {
    selectValue.value = []
  }

  const closeModal = () => {
    resetState()
    isModal.value = false
  }

  const subAuthRole = async () => {
    const params = {
      roleId: selectValue.value.join(','),
      userId: currentRow.userId
    }
    await apiUserRoleAuth(params)

    message.success('分配角色成功！')
    closeModal()
  }
  return {
    isModal,
    roleHandle,
    roleColumns,
    selectOption,
    selectValue,
    onSelectChange,
    authRole,
    closeModal,
    subAuthRole
  }
}
